!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!            PUT_STR_DENSITY_ON_OSOLVE    MARCH 2013                           |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine put_str_density_on_osolve(params,density_str,osolve)

use definitions

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine takes the density values for all string particles within an
! element, averages them and assigns them to the elemental compaction_density

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

implicit none

!include 'mpif.h'

type (parameters) params
type (string) :: density_str(2**params%levelmax_oc,2**params%levelmax_oct)
type(octreesolve) osolve

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

double precision :: xstr,ystr,zstr,x0,y0,z0,dxyz,zmax_elem
double precision :: str_length_inc
integer :: i,j,k,size_str(2),leaf,level,loc
logical :: next_elem

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------

! Zero out existing osolve%compaction_density
osolve%compaction_density=0.d0

! Could calculate this before calc_density_profile and pass in
size_str=size(density_str(1,1)%density)

! String length increment
str_length_inc = 1/(2.d0**dble(params%levelmax_oct))/dble(params%dstrnz)

do j=1,2**params%levelmax_oct
  do i=1,2**params%levelmax_oct
    ! Could calculate this before calc_density_profile and pass in
    xstr = 1/(2.d0**params%levelmax_oct)*(0.5+i-1)
    ystr = 1/(2.d0**params%levelmax_oct)*(0.5+j-1)

    next_elem = .true.
    do k=1,size_str(1)
      ! Note we're not accounting for squishing here...
      zstr = 1/(2.d0**params%levelmax_oct)*(0.5+k-1)/params%dstrnz
      if (next_elem) then
        call octree_find_leaf (osolve%octree,osolve%noctree,xstr,ystr,zstr,    &
                              leaf,level,loc,x0,y0,z0,dxyz)
        nstr_per_elem = int(2.d0**(2.d0*dble(params%levelmax_oct - level)))
        nstr_pts_per_elem = int(2.d0**(dble(params%levelmax_oct - level)) *    &
                            dble(nstr_per_elem) * dble(params%dstrnz))
        zmax_elem = z0 + dxyz / 2.d0
        next_elem = .false.
      endif
      osolve%compaction_density(leaf) = osolve%compaction_density(leaf) + density_str(i,j)%density(k)/dble(nstr_pts_per_elem)
      if (zmax_elem - zstr .le. str_length_inc) next_elem = .true.
    enddo
  enddo
enddo


!Note: if storing velocity at a differently sized array, could compare
! density, densityp outside the above loop over all instances of density_str



end subroutine put_str_density_on_osolve

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
